#!/usr/bin/env python3


import socket
import struct
import time
import numpy as np
from scipy import signal
from scipy.io import wavfile
import sys
from math import pi

SET_FREQUENCY = 0x01
SET_SAMPLERATE = 0x02
SET_GAINMODE = 0x03
SET_GAIN = 0x04
SET_FREQENCYCORRECTION = 0x05

class RtlTCPClient:
    def __init__(self):
        # Connect to a remote rtl_tcp server
        # TODO: Do it with config file
        self.remote_host = "office.toyaga.eu"
        self.remote_port = 51234
        self.conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.conn.connect((self.remote_host, self.remote_port))
        self.send_command(SET_SAMPLERATE, 20800)

    def tune(self, freq):
        # Tune to frequency
        self.send_command(SET_FREQUENCY, int(freq))

    def send_command(self, command, parameter):
        # Send command to server
        cmd = struct.pack(">BI", command, parameter)
        self.conn.send(cmd)

    def receive_data(self, duration):
        # Receive data from server
        raw_data = b""
        data_amount = 0
        second = time.time() 
        deadline = second + duration
        while True:
            
            try:
                if time.time() - second > 1:
                    second = time.time()
                    data_amount = 0
                data = self.conn.recv(1024)
                data_amount +=  len(data) 
                if not data:
                    raise ConnectionError("Connection closed by server")
                if time.time() >= deadline:
                    break
                
                raw_data += data
            except socket.timeout:
                break

        # Close the socket connection
        self.conn.close()

        # Turn the raw data into a numpy array
        iq = np.frombuffer(raw_data, dtype=np.uint16)
        
        # Unpack a sequence of bytes to a sequence of normalized complex numbers
        # iq = (iq - 127.5) / 128
        # iq = iq.view(np.complex64)
        # # Remove the leading 12 bytes that contain the header
        iq = iq[12:]
        return iq
    def demodulate_FM(self, iq):
        angles = np.angle(iq)

        # Determine phase rotation between samples
        # (Output one element less, that's we always save last sample
        # in remaining_data)
        rotations = np.ediff1d(angles)

        # Wrap rotations >= +/-180º
        rotations = (rotations + np.pi) % (2 * np.pi) - np.pi

        # Convert rotations to baseband signal 
        output_raw = np.multiply(rotations, 0.99 / (pi * 200000 / (20800 / 2)))
        output_raw = np.clip(output_raw, -0.999, +0.999)

        # Scale to signed 16-bit int
        output_raw = np.multiply(output_raw, 32767)
        output_raw = output_raw.astype(int)

        # Missing: low-pass filter and deemphasis filter
        # (result may be noisy)

        # Output as raw 16-bit, 1 channel audio
        bits = struct.pack(('<%dh' % len(output_raw)), *output_raw)

        # Write to raw file
        with open('fm.raw', 'wb') as f:
            f.write(bits)
        
        # return iq


    def demodulate_AM(self, iq):
        # https://stackoverflow.com/questions/61106688/rtl-sdr-iq-am-demodulation
        pass

        
